from django.db.models import Expression, fields


class RawMongoDBQuery(Expression):
    """Represent a wrapped raw mongodb query as a node within an expression."""

    query: dict

    def __init__(self, query, output_field=None):
        super().__init__(output_field=output_field)
        self.query = query

    def _resolve_output_field(self):
        return fields.BooleanField()



from django.db.backends.base.features import BaseDatabaseFeatures


class DatabaseFeatures(BaseDatabaseFeatures):
    supports_transactions = False
    supports_explaining_query_execution = False
    supports_json_field = True
    has_native_json_field = True
    supports_unlimited_charfield = True



import datetime

from bson import ObjectId
from django.db.backends.base.operations import BaseDatabaseOperations


class DatabaseOperations(BaseDatabaseOperations):
    compiler_module = "django_mongodb.compiler"

    def quote_name(self, name):
        if name.startswith('"') and name.endswith('"'):
            return name
        return f'"{name}"'

    def sql_flush(self, style, tables, *, reset_sequences=False, allow_cascade=False):
        return [{"op": "flush", "collection": table} for table in tables]

    def execute_sql_flush(self, sql_list):
        with self.connection.cursor() as conn:
            for sql in sql_list:
                if sql["op"] == "flush":
                    conn.connection.drop_collection(sql["collection"])

    def pk_default_value(self):
        return ObjectId()

    def adapt_json_value(self, value, encoder):
        return value

    def adapt_datetimefield_value(self, value):
        return value

    def adapt_timefield_value(self, value: datetime.time):
        if isinstance(value, datetime.time):
            return datetime.datetime.combine(datetime.date(2000, 1, 1), value)
        else:
            return value

    def adapt_datefield_value(self, value):
        if isinstance(value, datetime.date):
            return datetime.datetime(value.year, value.month, value.day)
        else:
            return value

    def convert_time_value(self, value, expression, connection):
        if isinstance(value, datetime.datetime):
            return value.time()
        return value

    def convert_date_value(self, value, expression, connection):
        if isinstance(value, datetime.datetime):
            return value.date()
        return value

    def get_db_converters(self, expression):
        converters = super().get_db_converters(expression)
        internal_type = expression.output_field.get_internal_type()
        match internal_type:
            case "TimeField":
                converters.append(self.convert_time_value)
            case "DateField":
                converters.append(self.convert_date_value)
        return converters



from django.conf import settings
from django.db.backends.base.creation import BaseDatabaseCreation


class DatabaseCreation(BaseDatabaseCreation):
    # mongodump --archive --db=test | mongorestore --archive  --nsFrom='test.*' --nsTo='examples.*'
    def create_test_db(self, verbosity=1, autoclobber=False, serialize=True, keepdb=False):
        test_database_name = self._get_test_db_name()
        if not keepdb:
            with self.connection._nodb_cursor() as cur:
                cur.drop_database(test_database_name)
        settings.DATABASES[self.connection.alias]["NAME"] = test_database_name
        return test_database_name

    def _destroy_test_db(self, test_database_name, verbosity):
        with self.connection._nodb_cursor() as cur:
            cur.drop_database(test_database_name)



from django.db.backends.base.schema import BaseDatabaseSchemaEditor


class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
    def quote_value(self, value):
        return value

    def add_constraint(self, model, constraint): ...

    def add_field(self, model, field): ...

    def add_index(self, model, index): ...

    def alter_db_table(self, model, old_db_table, new_db_table): ...

    def alter_db_table_comment(self, model, old_db_table_comment, new_db_table_comment): ...

    def alter_db_tablespace(self, model, old_db_tablespace, new_db_tablespace): ...

    def alter_field(self, model, old_field, new_field, strict=False): ...

    def alter_index_together(self, model, old_index_together, new_index_together): ...

    def alter_unique_together(self, model, old_unique_together, new_unique_together): ...

    def create_model(self, model): ...

    def delete_model(self, model): ...

    def effective_default(self, field): ...

    def prepare_default(self, value): ...

    def quote_name(self, name): ...

    def remove_constraint(self, model, constraint): ...

    def remove_field(self, model, field): ...

    def remove_index(self, model, index): ...

    def remove_procedure(self, procedure_name, param_types=()): ...

    def rename_index(self, model, old_index_name, new_index_name): ...

    def skip_default(self, field): ...

    def skip_default_on_alter(self, field): ...



from dataclasses import dataclass

from django.db.backends.base.introspection import BaseDatabaseIntrospection


@dataclass
class TableInfo:
    name: str
    type: str


class DatabaseIntrospection(BaseDatabaseIntrospection):
    def get_table_list(self, cursor):
        with self.connection.cursor() as conn:
            return [
                TableInfo(name=name, type="t") for name in conn.connection.list_collection_names()
            ]
